/*============================================================================
  Summary:  Demonstrates how to analyze the Buffer Pool in SQL Server.
------------------------------------------------------------------------------
  Written by Klaus Aschenbrenner, www.SQLpassion.at

  For more information about SQL Server, check out my website
    http://www.SQLpassion.at
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

-- This session option tells you how many logical and physical reads a
-- specific query needed during its execution
SET STATISTICS IO ON
GO

-- Let's work with some DWH queries
USE ContosoRetailDW
GO

-- This query reports 44249 logical reads.
-- We have read 44249 pages of 8kb (around 345 MB of data).
SELECT COUNT(*) FROM FactOnlineSales
GO

-- Let's review the Buffer Pool.
SELECT * FROM sys.dm_os_buffer_descriptors
GO

-- Let's check how much data we currently store in the Buffer Pool
SELECT COUNT(*) * 8192 / 1024 / 1024 AS 'MB' FROM sys.dm_os_buffer_descriptors
GO

-- Now we want to break down which database takes
-- how much space in the Buffer Pool.
-- ContosoRetailDW caches around 350 MB in the Buffer Pool.
SELECT
	DB_NAME(database_id) AS 'DatabaseName',
	COUNT(*) * 8192 / 1024 / 1024 AS 'MB'
FROM sys.dm_os_buffer_descriptors
WHERE database_id <> 32767
GROUP BY database_id
ORDER BY COUNT(*) DESC
GO

-- Let's make a break down at the table level within ContosoRetailDW
SELECT * FROM sys.dm_os_buffer_descriptors
WHERE database_id = DB_ID('ContosoRetailDW')
GO

-- We can use the allocation_unit_id column to navigate up to the
-- table level through the meta data DMVs
SELECT
	OBJECT_NAME(p.object_id) AS 'TableName',
	COUNT(*) * 8192 / 1024 AS 'KB'
FROM sys.dm_os_buffer_descriptors bd
INNER JOIN sys.allocation_units au ON au.allocation_unit_id = bd.allocation_unit_id
INNER JOIN sys.partitions p ON p.partition_id = au.container_id
INNER JOIN sys.tables t ON t.object_id = p.object_id
WHERE
	database_id = DB_ID('ContosoRetailDW')
	AND t.is_ms_shipped = 0 -- Excluces system tables
GROUP BY p.object_id
ORDER BY COUNT(*) DESC
GO